@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2016 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -19,30 +19,46 @@
 
 """Other utilities which don't fit anywhere else."""
 
+import os
 import io
+import re
 import sys
 import enum
 import json
 import os.path
 import collections
+import datetime
+import traceback
 import functools
 import contextlib
-import itertools
 import socket
 import shlex
 
+import attr
 from PyQt5.QtCore import Qt, QUrl
 from PyQt5.QtGui import QKeySequence, QColor, QClipboard, QDesktopServices
 from PyQt5.QtWidgets import QApplication
 import pkg_resources
+import yaml
+try:
+    from yaml import CSafeLoader as YamlLoader, CSafeDumper as YamlDumper
+    YAML_C_EXT = True
+except ImportError:  # pragma: no cover
+    from yaml import SafeLoader as YamlLoader, SafeDumper as YamlDumper
+    YAML_C_EXT = False
 
 import qutebrowser
-from qutebrowser.utils import qtutils, log
+from qutebrowser.utils import qtutils, log, debug
 
 
 fake_clipboard = None
 log_clipboard = False
 
+is_mac = sys.platform.startswith('darwin')
+is_linux = sys.platform.startswith('linux')
+is_windows = sys.platform.startswith('win')
+is_posix = os.name == 'posix'
+
 
 class ClipboardError(Exception):
 
@@ -53,6 +69,10 @@ class SelectionUnsupportedError(ClipboardError):
 
     """Raised if [gs]et_clipboard is used and selection=True is unsupported."""
 
+    def __init__(self):
+        super().__init__("Primary selection is not supported on this "
+                         "platform!")
+
 
 class ClipboardEmptyError(ClipboardError):
 
@@ -129,7 +149,8 @@ def read_file(filename, binary=False):
         The file contents as string.
     """
     if hasattr(sys, 'frozen'):
-        # cx_Freeze doesn't support pkg_resources :(
+        # PyInstaller doesn't support pkg_resources :(
+        # https://github.com/pyinstaller/pyinstaller/wiki/FAQ#misc
         fn = os.path.join(os.path.dirname(sys.executable), filename)
         if binary:
             with open(fn, 'rb') as f:
@@ -158,37 +179,6 @@ def resource_filename(filename):
     return pkg_resources.resource_filename(qutebrowser.__name__, filename)
 
 
-def actute_warning():
-    """Display a warning about the dead_actute issue if needed."""
-    # WORKAROUND (remove this when we bump the requirements to 5.3.0)
-    # Non Linux OS' aren't affected
-    if not sys.platform.startswith('linux'):
-        return
-    # If no compose file exists for some reason, we're not affected
-    if not os.path.exists('/usr/share/X11/locale/en_US.UTF-8/Compose'):
-        return
-    # Qt >= 5.3 doesn't seem to be affected
-    try:
-        if qtutils.version_check('5.3.0'):
-            return
-    except ValueError:  # pragma: no cover
-        pass
-    try:
-        with open('/usr/share/X11/locale/en_US.UTF-8/Compose', 'r',
-                  encoding='utf-8') as f:
-            for line in f:
-                if '<dead_actute>' in line:
-                    if sys.stdout is not None:
-                        sys.stdout.flush()
-                    print("Note: If you got a 'dead_actute' warning above, "
-                          "that is not a bug in qutebrowser! See "
-                          "https://bugs.freedesktop.org/show_bug.cgi?id=69476 "
-                          "for details.")
-                    break
-    except OSError:
-        log.init.exception("Failed to read Compose file")
-
-
 def _get_color_percentage(a_c1, a_c2, a_c3, b_c1, b_c2, b_c3, percent):
     """Get a color which is percent% interpolated between start and end.
 
@@ -397,9 +387,9 @@ def keyevent_to_string(e):
         A name of the key (combination) as a string or
         None if only modifiers are pressed..
     """
-    if sys.platform == 'darwin':
-        # Qt swaps Ctrl/Meta on OS X, so we switch it back here so the user can
-        # use it in the config as expected. See:
+    if is_mac:
+        # Qt swaps Ctrl/Meta on macOS, so we switch it back here so the user
+        # can use it in the config as expected. See:
         # https://github.com/qutebrowser/qutebrowser/issues/110
         # http://doc.qt.io/qt-5.4/osx-issues.html#special-keys
         modmask2str = collections.OrderedDict([
@@ -427,9 +417,10 @@ def keyevent_to_string(e):
         if mod & mask and s not in parts:
             parts.append(s)
     parts.append(key_to_string(e.key()))
-    return '+'.join(parts)
+    return normalize_keystr('+'.join(parts))
 
 
+@attr.s(repr=False)
 class KeyInfo:
 
     """Stores information about a key, like used in a QKeyEvent.
@@ -440,26 +431,20 @@ class KeyInfo:
         text: str
     """
 
-    def __init__(self, key, modifiers, text):
-        self.key = key
-        self.modifiers = modifiers
-        self.text = text
+    key = attr.ib()
+    modifiers = attr.ib()
+    text = attr.ib()
 
     def __repr__(self):
-        # Meh, dependency cycle...
-        from qutebrowser.utils.debug import qenum_key
         if self.modifiers is None:
             modifiers = None
         else:
             #modifiers = qflags_key(Qt, self.modifiers)
             modifiers = hex(int(self.modifiers))
-        return get_repr(self, constructor=True, key=qenum_key(Qt, self.key),
+        return get_repr(self, constructor=True,
+                        key=debug.qenum_key(Qt, self.key),
                         modifiers=modifiers, text=self.text)
 
-    def __eq__(self, other):
-        return (self.key == other.key and self.modifiers == other.modifiers and
-                self.text == other.text)
-
 
 class KeyParseError(Exception):
 
@@ -762,25 +747,6 @@ def sanitize_filename(name, replacement='_'):
     return name
 
 
-def newest_slice(iterable, count):
-    """Get an iterable for the n newest items of the given iterable.
-
-    Args:
-        count: How many elements to get.
-               0: get no items:
-               n: get the n newest items
-              -1: get all items
-    """
-    if count < -1:
-        raise ValueError("count can't be smaller than -1!")
-    elif count == 0:
-        return []
-    elif count == -1 or len(iterable) < count:
-        return iterable
-    else:
-        return itertools.islice(iterable, len(iterable) - count, len(iterable))
-
-
 def set_clipboard(data, selection=False):
     """Set the clipboard to some given data."""
     if selection and not supports_selection():
@@ -793,11 +759,23 @@ def set_clipboard(data, selection=False):
         QApplication.clipboard().setText(data, mode=mode)
 
 
-def get_clipboard(selection=False):
-    """Get data from the clipboard."""
+def get_clipboard(selection=False, fallback=False):
+    """Get data from the clipboard.
+
+    Args:
+        selection: Use the primary selection.
+        fallback: Fall back to the clipboard if primary selection is
+                  unavailable.
+    """
     global fake_clipboard
+    if fallback and not selection:
+        raise ValueError("fallback given without selection!")
+
     if selection and not supports_selection():
-        raise SelectionUnsupportedError
+        if fallback:
+            selection = False
+        else:
+            raise SelectionUnsupportedError
 
     if fake_clipboard is not None:
         data = fake_clipboard
@@ -831,27 +809,28 @@ def random_port():
 def open_file(filename, cmdline=None):
     """Open the given file.
 
-    If cmdline is not given, general->default-open-dispatcher is used.
-    If default-open-dispatcher is unset, the system's default application is
-    used.
+    If cmdline is not given, downloads.open_dispatcher is used.
+    If open_dispatcher is unset, the system's default application is used.
 
     Args:
         filename: The filename to open.
         cmdline: The command to use as string. A `{}` is expanded to the
                  filename. None means to use the system's default application
-                 or `default-open-dispatcher` if set. If no `{}` is found, the
-                 filename is appended to the cmdline.
+                 or `downloads.open_dispatcher` if set. If no `{}` is found,
+                 the filename is appended to the cmdline.
     """
     # Import late to avoid circular imports:
-    # utils -> config -> configdata -> configtypes -> cmdutils -> command ->
-    # utils
-    from qutebrowser.misc import guiprocess
+    # - usertypes -> utils -> guiprocess -> message -> usertypes
+    # - usertypes -> utils -> config -> configdata -> configtypes ->
+    #   cmdutils -> command -> message -> usertypes
     from qutebrowser.config import config
+    from qutebrowser.misc import guiprocess
+
     # the default program to open downloads with - will be empty string
     # if we want to use the default
-    override = config.get('general', 'default-open-dispatcher')
+    override = config.val.downloads.open_dispatcher
 
-    # precedence order: cmdline > default-open-dispatcher > openUrl
+    # precedence order: cmdline > downloads.open_dispatcher > openUrl
 
     if cmdline is None and not override:
         log.misc.debug("Opening {} with the system application"
@@ -876,3 +855,57 @@ def open_file(filename, cmdline=None):
 def unused(_arg):
     """Function which does nothing to avoid pylint complaining."""
     pass
+
+
+def expand_windows_drive(path):
+    r"""Expand a drive-path like E: into E:\.
+
+    Does nothing for other paths.
+
+    Args:
+        path: The path to expand.
+    """
+    # Usually, "E:" on Windows refers to the current working directory on drive
+    # E:\. The correct way to specifify drive E: is "E:\", but most users
+    # probably don't use the "multiple working directories" feature and expect
+    # "E:" and "E:\" to be equal.
+    if re.match(r'[A-Z]:$', path, re.IGNORECASE):
+        return path + "\\"
+    else:
+        return path
+
+
+def yaml_load(f):
+    """Wrapper over yaml.load using the C loader if possible."""
+    start = datetime.datetime.now()
+    data = yaml.load(f, Loader=YamlLoader)
+    end = datetime.datetime.now()
+
+    delta = (end - start).total_seconds()
+    deadline = 3 if 'CI' in os.environ else 1
+    if delta > deadline:  # pragma: no cover
+        log.misc.warning(
+            "YAML load took unusually long, please report this at "
+            "https://github.com/qutebrowser/qutebrowser/issues/2777\n"
+            "duration: {}s\n"
+            "PyYAML version: {}\n"
+            "C extension: {}\n"
+            "Stack:\n\n"
+            "{}".format(
+                delta, yaml.__version__, YAML_C_EXT,
+                ''.join(traceback.format_stack())))
+
+    return data
+
+
+def yaml_dump(data, f=None):
+    """Wrapper over yaml.dump using the C dumper if possible.
+
+    Also returns a str instead of bytes.
+    """
+    yaml_data = yaml.dump(data, f, Dumper=YamlDumper, default_flow_style=False,
+                          encoding='utf-8', allow_unicode=True)
+    if yaml_data is None:
+        return None
+    else:
+        return yaml_data.decode('utf-8')
